unit LogServer;

interface

uses
  Classes, StdCtrls, IdUdpServer, IdUdpClient, IdSocketHandle, Logger;

type
  TLogServer = class(TComponent)
  private
    FFileName: String;
    FTeamName: String;
    FPlayerNumber: String;
    FServerHost: String;
    FClientHost: String;
    FClientInit: String;
    FServerPort: Integer;
    FClientPort: Integer;
    FBufferSize: Integer;
    FServerSocket: TIdUdpServer;
    FClientSocket: TIdUdpServer;
    FMemoOutput: TMemo;
    FLogFile: TLogger;

    procedure ActivateServerSocket(LocalPort: Integer; RemoteHost: String; RemotePort: Integer);
    procedure ActivateClientSocket(LocalPort: Integer; RemoteHost: String; RemotePort: Integer);
    procedure ParseClientInit(Data: String);
    procedure ParseServerInit(Data: String);
    procedure StartServerInitialization(InitData: String);
    procedure OnServerReceiveEvent(Sender: TObject; AData: TStream; ABinding: TIdSocketHandle);
    procedure OnClientReceiveEvent(Sender: TObject; AData: TStream; ABinding: TIdSocketHandle);
  public
    constructor Create(output: TMemo);
    destructor Destroy; override;

    procedure StopCommunication;
    procedure Initialize(Data: String; Binding: TIdSocketHandle;
                    ServerHost: String; ServerPort: Integer; StartPort: Integer);
  end; { TLogServer }

implementation

uses
  Forms, Dialogs, SysUtils;

const
    MIN_PORT = 10001;
    MAX_PORT = 13999;
    OLD_MSG_SIZE = 1024;
    MSG_SIZE = 4096;
    FILE_EXTENSION = '.lsf';

{ TLogServer }

constructor TLogServer.Create(output: TMemo);
begin
    inherited Create(nil);

    FMemoOutput := output;
    FServerHost := '127.0.0.1';
    FServerPort := 6000;
    FBufferSize := OLD_MSG_SIZE;
    FServerSocket := nil;
    FClientSocket := nil;
    FLogFile := nil;
    FFileName := '';
    FTeamName := '';
    FPlayerNumber := '';
end; { TLogServer.Create }

destructor TLogServer.Destroy;
begin
    // Close and Free memory for the sockets
    StopCommunication;
    if Assigned(FServerSocket) then
        FServerSocket.Free;
    if Assigned(FClientSocket) then
        FClientSocket.Free;
    if Assigned(FLogFile) then
        FLogFile.Free;

    inherited Destroy;
end; { TLogServer.Destroy }

procedure TLogServer.Initialize(Data: String; Binding: TIdSocketHandle;
  ServerHost: String; ServerPort: Integer; StartPort: Integer);
begin
    // Initialize LogServer

    // Activate and open socket communication channels
    if (Pos('(init', Data) = 1) then
    begin
        FClientHost := Binding.PeerIP;
        FClientPort := Binding.PeerPort;
        ParseClientInit(Trim(Data));
        ActivateClientSocket(StartPort, Binding.PeerIP, Binding.PeerPort);
        ActivateServerSocket(StartPort + 2, ServerHost, ServerPort);
        StartServerInitialization(Trim(Data));
    end; // if (Pos('(init', Data) = 1) then
end; { TLogServer.Initialize }

procedure TLogServer.ParseClientInit(Data: String);
var
    Version: Double;
    TempData: String;
    VersionPos: Integer;
begin
    // Parse Init message
    FClientInit := Data;
    Version := 3.0;

    TempData := Trim(Copy(Data, 6, Length(Data)));

    if (Pos(' ', TempData) > 0) then
        FTeamName := Trim(Copy(TempData, 0, Pos(' ', TempData) - 1))
    else
        FTeamName := Trim(Copy(TempData, 0, Pos(')', TempData) - 1));

    // Parse out the Protocol version number if it has been supplied
    VersionPos := Pos('(version', TempData);
    if (VersionPos > 0) then
    begin
        TempData := Trim(Copy(TempData, VersionPos + 9, Length(TempData)));
        Version := StrToFloat(Trim(Copy(TempData, 0, Pos(')', TempData) - 1)));
    end; // if (Pos('(version', Data) > 0) then

    // Set the relevant buffer size depending on the version protocol number
    if (Version >= 6.0) then
        FBufferSize := MSG_SIZE
    else
        FBufferSize := OLD_MSG_SIZE;

    Application.ProcessMessages;
end; { TLogServer.ParseInit }

procedure TLogServer.ParseServerInit(Data: String);
var
    TempData: String;
begin
    // Obtain the Player number from the server's Init message
    TempData := Trim(Copy(Data, 8, Length(Data)));
    FPlayerNumber := Trim(Copy(TempData, 0, Pos(' ', TempData) - 1));

    // Prepend 0 to the player number if it is below 10.
    if (StrToInt(FPlayerNumber) < 10) then
        FPlayerNumber := '0' + FPlayerNumber;

    // Set FFileName for the Log File
    if ((FTeamName <> '') and (FPlayerNumber <> '')) then
    begin
        FFileName := FTeamName + '_' + FPlayerNumber + FILE_EXTENSION;
        FLogFile := TLogger.Create(FFileName);
    end; // if (FFileName <> '') then
end; { TLogServer.ParseInit }

procedure TLogServer.StopCommunication;
begin
    // Stop sockets
    if Assigned(FClientSocket) then
    begin
        if FCLientSocket.Active then
            FClientSocket.Active := False;
    end; // if Assigned(FClientSocket) then
    if Assigned(FServerSocket) then
    begin
        if FServerSocket.Active then
            FServerSocket.Active := False;
    end; // if Assigned(FServerSocket) then
end; { TLogServer.StopCommunication }

procedure TLogServer.ActivateClientSocket(LocalPort: Integer;
  RemoteHost: String; RemotePort: Integer);
begin
    // Activate Client Socket
    FClientSocket := TIdUdpServer.Create(Self);
    with FClientSocket do
    begin
        OnUDPRead := OnClientReceiveEvent;
        DefaultPort := LocalPort;
        BufferSize := FBufferSize;
        Active := True;
    end; // with FClientSocket do

    FClientHost := RemoteHost;
    FClientPort := RemotePort;
end; { TLogServer.ActivateClientSocket }

procedure TLogServer.ActivateServerSocket(LocalPort: Integer;
  RemoteHost: String; RemotePort: Integer);
begin
    // Activate Server Socket
    FServerSocket := TIdUdpServer.Create(Self);
    with FServerSocket do
    begin
        OnUDPRead := OnServerReceiveEvent;
        DefaultPort := LocalPort;
        BufferSize := FBufferSize;
        Active := True;
    end; // with FClientSocket do

    FServerHost := RemoteHost;
    FServerPort := RemotePort;
end; { TLogServer.ActivateServerSocket }

procedure TLogServer.OnClientReceiveEvent(Sender: TObject; AData: TStream;
  ABinding: TIdSocketHandle);
var
    DataStringStream: TStringStream;
    Data: String;
begin
    // On receive of UDP data packet from the client
    DataStringStream := TStringStream.Create('');
    try
        DataStringStream.CopyFrom(AData, AData.Size);
        Data := DataStringStream.DataString;
        FServerSocket.Send(FServerHost, FServerPort, Data);
        FLogFile.AddRecord(Data);
    finally
        DataStringStream.Free;
    end;
    Application.ProcessMessages;
end; { TLogServer.OnClientReceiveEvent }

procedure TLogServer.OnServerReceiveEvent(Sender: TObject; AData: TStream;
  ABinding: TIdSocketHandle);
var
    DataStringStream: TStringStream;
    Data: String;
begin
    // On Receive of information
    DataStringStream := TStringStream.Create('');
    try
        DataStringStream.CopyFrom(AData, AData.Size);
        Data := Trim(DataStringStream.DataString);

        if (Pos('(init', Data) = 1) then
        begin
            FServerHost := ABinding.PeerIP;
            FServerPort := ABinding.PeerPort;
            ParseServerInit(Data);
            FLogFile.AddRecord(FClientInit);
            FMemoOutput.Lines.Add('Logging initialized for ' + FTeamName +
                    ', player ' + FPlayerNumber + ' --> ' + FFileName);
        end; // if (Pos('(init', Data) = 1) then

        // Route data from the server to the client
        FClientSocket.Send(FClientHost, FClientPort, Data);
        FLogFile.AddRecord(Data);
    finally
        DataStringStream.Free;
    end;
    Application.ProcessMessages;
end; { TLogServer.OnServerReceiveEvent }

procedure TLogServer.StartServerInitialization(InitData: String);
begin
    // The entire process begins by simply forwarding the
    // init statement from the Client to the Server
    FServerSocket.Send(FServerHost, FServerPort, InitData);
end; { TLogServer.StartServerInitialization }

end.
